=== modified file 'NEWS'
--- NEWS	2005-10-17 07:48:02 +0000
+++ NEWS	2005-10-18 01:13:21 +0000
@@ -2,6 +2,10 @@
 
   IMPROVEMENTS:
 
+    * New 'bzr commit --strict' option refuses to commit if there are 
+      any unknown files in the tree.  To commit, make sure all files are 
+      either ignored, added, or deleted.  (Michael Ellerman)
+
     * The config directory is now ~/.bazaar, and there is a single file 
       ~/.bazaar/bazaar.conf storing email, editor and other preferences.
       (Robert Collins)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-17 06:28:03 +0000
+++ bzrlib/builtins.py	2005-10-18 01:13:21 +0000
@@ -1006,7 +1006,9 @@
     """
     # TODO: Run hooks on tree to-be-committed, and after commit.
 
-    # TODO: Strict commit that fails if there are unknown or deleted files.
+    # TODO: Strict commit that fails if there are deleted files.
+    #       (what does "deleted files" mean ??)
+
     # TODO: Give better message for -s, --summary, used by tla people
 
     # XXX: verbose currently does nothing
@@ -1018,12 +1020,16 @@
                      Option('file', type=str, 
                             argname='msgfile',
                             help='file containing commit message'),
+                     Option('strict',
+                            help="refuse to commit if there are unknown "
+                            "files in the working tree."),
                      ]
     aliases = ['ci', 'checkin']
 
     def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit, ConflictsInTree
+            unchanged=False, strict=False):
+        from bzrlib.errors import (PointlessCommit, ConflictsInTree,
+                StrictCommitFailed)
         from bzrlib.msgeditor import edit_commit_message
         from bzrlib.status import show_status
         from cStringIO import StringIO
@@ -1052,9 +1058,8 @@
                 raise BzrCommandError("empty commit message specified")
             
         try:
-            b.commit(message,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
+            b.commit(message, specific_files=selected_list,
+                     allow_pointless=unchanged, strict=strict)
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
@@ -1063,6 +1068,9 @@
         except ConflictsInTree:
             raise BzrCommandError("Conflicts detected in working tree.  "
                 'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+        except StrictCommitFailed:
+            raise BzrCommandError("Commit refused because there are unknown "
+                                  "files in the working tree.")
 
 
 class cmd_check(Command):

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-15 11:38:29 +0000
+++ bzrlib/commit.py	2005-10-18 01:13:21 +0000
@@ -75,7 +75,8 @@
 import bzrlib.config
 from bzrlib.errors import (BzrError, PointlessCommit,
                            HistoryMissing,
-                           ConflictsInTree
+                           ConflictsInTree,
+                           StrictCommitFailed
                            )
 from bzrlib.revision import Revision
 from bzrlib.trace import mutter, note, warning
@@ -160,6 +161,7 @@
                specific_files=None,
                rev_id=None,
                allow_pointless=True,
+               strict=False,
                verbose=False,
                revprops=None):
         """Commit working copy as a new revision.
@@ -178,6 +180,9 @@
         allow_pointless -- If true (default), commit even if nothing
             has changed and no merges are recorded.
 
+        strict -- If true, don't allow a commit if the working tree
+            contains unknown files.
+
         revprops -- Properties for new revision
         """
         mutter('preparing to commit')
@@ -189,6 +194,9 @@
         self.allow_pointless = allow_pointless
         self.revprops = revprops
 
+        if strict and branch.unknowns():
+            raise StrictCommitFailed()
+
         if timestamp is None:
             self.timestamp = time.time()
         else:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-10-17 09:39:46 +0000
+++ bzrlib/errors.py	2005-10-18 01:13:21 +0000
@@ -125,6 +125,8 @@
     def __str__(self):
         return self.args[0]
 
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
 
 class NotBranchError(BzrNewError):
     """Not a branch: %(path)s"""

=== modified file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	2005-10-06 07:03:33 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-18 01:13:21 +0000
@@ -246,3 +246,21 @@
         self.assertEqual('1', inv['file1id'].revision)
         # FIXME: This should raise a KeyError I think, rbc20051006
         self.assertRaises(BzrError, inv.__getitem__, 'file2id')
+
+    def test_strict_commit(self):
+        """Try and commit with unknown files and strict = True, should fail."""
+        from bzrlib.errors import StrictCommitFailed
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        self.assertRaises(StrictCommitFailed, b.commit,
+            message='add hello but not goodbye', strict=True)
+
+    def test_nonstrict_commit(self):
+        """Try and commit with unknown files and strict = False, should work."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        b.commit(message='add hello but not goodbye', strict=False)

